@@ -23,14 +23,15 @@ from telethon.tl.types import InputPeerEmpty, PeerUser, PeerChat, PeerChannel, P
 from telethon.tl.functions.messages import GetDialogsRequest
 from telethon import TelegramClient, functions, types, utils
 from telethon.utils import get_display_name, get_message_id
-from telethon.sync import TelegramClient
+from googletrans import Translator, constants
 from colorama import Fore, Back, Style
 from alive_progress import alive_bar
 from bs4 import BeautifulSoup
+from pprint import pprint
 
 __author__ = "Jordan Wildon (@jordanwildon)"
 __license__ = "MIT License"
-__version__ = "2.1.9"
+__version__ = "2.1.10"
 __maintainer__ = "Jordan Wildon"
 __email__ = "j.wildon@pm.me"
 __status__ = "Development"
@@ -87,7 +88,7 @@ user_agent = [
 ]
 
 @click.command()
-@click.option('--target', '-t', default = '', multiple = True,
+@click.option('--target', '-t', default = ' ', multiple = True,
               help = 'Specifies a chat to investigate.')
 @click.option('--comprehensive', '-c', is_flag = True,
               help = 'Comprehensive scan, includes archiving.')
@@ -95,17 +96,18 @@ user_agent = [
               help = 'Archives media in the specified chat.')
 @click.option('--forwards', '-f', is_flag = True,
               help = 'Scrapes forwarded messages.')
-@click.option('--user', '-u', default = '',
+@click.option('--user', '-u', is_flag = True,
             help = 'Looks up a specified user ID.')
-@click.option('--location', '-l', default = '',
+@click.option('--location', '-l', is_flag = True,
             help = 'Finds users near to specified coordinates.')
 @click.option('--alt', '-a', is_flag = True, default = False,
             help = 'Uses an alternative login.')
 @click.option('--json', '-j', is_flag = True, default = False,
-            help = 'Export to JSON')
+            help = 'Export to JSON.')
+@click.option('--export', '-e', is_flag = True, default = False,
+            help = 'Export a list of chats your account is part of.')
 
-
-def cli(target,comprehensive,media,forwards,user,location,alt,json):
+def cli(target,comprehensive,media,forwards,user,location,alt,json,export):
 
     print(Fore.GREEN + """
       ______     __                 __  __
@@ -115,7 +117,7 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
     /_/  \___/_/\___/ .___/\__,_/\__/_/ /_/\__, /
                    /_/                    /____/
     -- An OSINT toolkit for investigating Telegram chats.
-    -- Developed by @jordanwildon | Version 2.1.9.
+    -- Developed by @jordanwildon | Version 2.1.10.
     """)
 
     print(Style.RESET_ALL)
@@ -126,6 +128,33 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
     except FileExistsError:
         pass
 
+    # Defining default values
+    basic = False
+    comp_check = False
+    media_archive = False
+    forwards_check = False
+    forward_verify = False
+    user_check = False
+    location_check = False
+    last_date = None
+    chunk_size = 200
+    forwards_check = False
+
+    #Will add more languages later
+    user_language = "en"
+
+    if forwards:
+        forwards_check = True
+    if user:
+        user_check = True
+        basic = False
+    if location:
+        location_check = True
+        basic = False
+    if comprehensive:
+        comp_check = True
+    if media:
+        media_archive = True
     if alt:
         alt_check = True
     else:
@@ -189,83 +218,63 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
         # Creating logfile
         log_file = telepathy_file + 'log.csv'
 
-        # Defining default values
-        basic = False
-        comp_check = False
-        media_archive = False
-        forward_check = False
-        forward_verify = False
-        user_check = False
-        location_check = False
-        last_date = None
-        chunk_size = 200
-
         # Connecting to the Telegram client and defining options
-        client.connect()
-        if not client.is_user_authorized():
-            client.send_code_request(phone_number)
-            client.sign_in(phone_number)
-            try:
-                client.sign_in(code=input(' Enter code: '))
-            except SessionPasswordNeededError:
-                client.sign_in(password=getpass.getpass(
-                    prompt='Password: ', stream=None))
-
-            result = client(GetDialogsRequest(
-                            offset_date = last_date,
-                            offset_id = 0,
-                            offset_peer = InputPeerEmpty(),
-                            limit = chunk_size,
-                            hash = 0
-                            ))
-        else:
-            if not comprehensive and not user and not location:
-                basic = True
-                comp_check = False
-                print(Fore.GREEN + ' [!] '
-                      + Style.RESET_ALL
-                      + 'Performing basic scan')
-            elif user:
-                user_check = True
-                t = user
-            elif location:
-                location_check = True
-                t = location
-            elif comprehensive:
-                print(Fore.GREEN + ' [!] '
-                      + Style.RESET_ALL
-                      + 'Performing comprehensive scan')
-
-                file_archive = save_directory + '/' + alphanumeric + '_' + filetime_clean + '_archive.csv'
-                file_forwards = save_directory + '/edgelists/' + alphanumeric + '_' + filetime_clean + '_edgelist.csv'
-
-                comp_check = True
-
-            if forwards:
-                print(Fore.GREEN + ' [!] '
-                      + Style.RESET_ALL
-                      + 'Forwards will be fetched')
-
-                forward_directory = save_directory + '/edgelists/'
+        async def main():
 
+            await client.connect()
+            if not await client.is_user_authorized():
+                client.send_code_request(phone_number)
+                client.sign_in(phone_number)
                 try:
-                    os.makedirs(forward_directory)
-                except FileExistsError:
-                    pass
+                    client.sign_in(code=input(' Enter code: '))
+                except SessionPasswordNeededError:
+                    client.sign_in(password=getpass.getpass(
+                        prompt='Password: ', stream=None))
 
-                edgelist_file = forward_directory + '/' + alphanumeric + '_edgelist.csv'
-                forward_check = True
+                result = client(GetDialogsRequest(
+                                offset_date = last_date,
+                                offset_id = 0,
+                                offset_peer = InputPeerEmpty(),
+                                limit = chunk_size,
+                                hash = 0
+                                ))
+            else:
+                if basic == True:
+                    print(Fore.GREEN + ' [!] '
+                          + Style.RESET_ALL
+                          + 'Performing basic scan')
 
-            if basic is True or comp_check is True:
+                elif comp_check == True:
+                    print(Fore.GREEN + ' [!] '
+                          + Style.RESET_ALL
+                          + 'Performing comprehensive scan')
 
-                print('\n' + Fore.GREEN
-                      + ' [-] '
-                      + Style.RESET_ALL
-                      + 'Fetching details for '
-                      + t
-                      + '...')
+                    file_archive = save_directory + '/' + alphanumeric + '_' + filetime_clean + '_archive.csv'
 
-                async def main():
+                if forwards_check == True:
+                    print(Fore.GREEN + ' [!] '
+                          + Style.RESET_ALL
+                          + 'Forwards will be fetched')
+
+                    file_forwards = save_directory + '/edgelists/' + alphanumeric + '_' + filetime_clean + '_edgelist.csv'
+
+                    forward_directory = save_directory + '/edgelists/'
+
+                    try:
+                        os.makedirs(forward_directory)
+                    except FileExistsError:
+                        pass
+
+                    edgelist_file = forward_directory + '/' + alphanumeric + '_edgelist.csv'
+
+                if basic is True or comp_check is True:
+
+                    print('\n' + Fore.GREEN
+                          + ' [-] '
+                          + Style.RESET_ALL
+                          + 'Fetching details for '
+                          + t
+                          + '...')
 
                     memberlist_directory = save_directory + '/memberlists'
 
@@ -290,7 +299,7 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
 
                         date = str(year) + "/" + str(month) + "/" + str(day)
                         mtime = str(hour) + ":" + str(minute) + ":" + str(second)
-                        first_post = date + "T" + mtime + '+00:00 UTC'
+                        first_post = date + " " + mtime + ' +00:00 UTC'
                         break
 
                     if entity.username:
@@ -388,8 +397,8 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                             full_name = (first_name + ' ' + last_name).strip()
 
                             members_df = pd.DataFrame(members, columns = ['username','full name',
-                                                                  'user id','phone number',
-                                                                  'group name'])
+                                                                          'user id','phone number',
+                                                                          'group name'])
 
                             members.append([username,full_name,user.id,phone,t])
 
@@ -496,9 +505,9 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                     if not os.path.isfile(log_file):
                        log_df.to_csv(log_file, sep = ';', index = False)
                     else:
-                       log_df.to_csv(log_file, mode='a', index = False, header=False)
+                       log_df.to_csv(log_file, sep = ';', mode='a', index = False, header=False)
 
-                    if forward_check is True and comp_check is False:
+                    if forwards_check is True and comp_check is False:
                         print(Fore.GREEN
                               + ' [-] '
                               + Style.RESET_ALL
@@ -677,7 +686,9 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                                       + Style.RESET_ALL
                                       + 'Media content will be archived')
 
-                            print(Fore.GREEN + ' [!] '
+                            print(Fore.GREEN
+                                  + '\n'
+                                  + ' [!] '
                                   + Style.RESET_ALL
                                   + 'Calculating number of messages...')
 
@@ -709,8 +720,9 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                                             c_archive = pd.DataFrame(message_list,
                                                                      columns = ['To', 'Message ID',
                                                                                  'Display_name', 'ID',
-                                                                                 'Message_text', 'Timestamp',
-                                                                                 'Reply', 'Views'])
+                                                                                 'Message_text', 'Original_language',
+                                                                                 'Translated_text', 'Translation_confidence',
+                                                                                 'Timestamp', 'Reply', 'Views'])
 
                                             c_forwards = pd.DataFrame(forwards_list,
                                                                       columns = ['To', 'To_title',
@@ -718,7 +730,16 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                                                                                 'Timestamp'])
 
                                             display_name = get_display_name(message.sender)
-                                            nameID = message.from_id
+                                            if chat_type != 'Channel':
+                                                substring = "PeerUser"
+                                                string = str(message.from_id)
+                                                if substring in string:
+                                                    user_id = re.sub("[^0-9]", "", string)
+                                                    nameID = str(user_id)
+                                                else:
+                                                    nameID = str(message.from_id)
+                                            else:
+                                                nameID = to_ent.id
                                             year = str(format(message.date.year, '02d'))
                                             month = str(format(message.date.month, '02d'))
                                             day = str(format(message.date.day, '02d'))
@@ -747,10 +768,26 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                                             else:
                                                 views = 'Not found'
 
+                                                #translate
+                                            if message_text != 'none':
+                                                translator = Translator()
+                                                detection = translator.detect(message_text)
+                                                language_code = detection.lang
+                                                translation_confidence = detection.confidence
+
+                                                translation = translator.translate(message_text, dest=user_language)
+                                                original_language = translation.src
+                                                translated_text = translation.text
+                                            else:
+                                                original_langauge = user_language
+                                                translated_text = 'n/a'
+                                                translation_confidence = 'n/a'
+
                                             message_list.append([t, message.id,
                                                                  display_name, nameID,
-                                                                 message_text, timestamp,
-                                                                 reply, views])
+                                                                 message_text, original_language,
+                                                                 translated_text, translation_confidence,
+                                                                 timestamp,reply, views])
 
                                             if message.forward is not None:
                                                 forward_verify = True
@@ -844,7 +881,7 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                             else:
                                 pass
 
-                            if forward_check is True:
+                            if forwards_check is True:
                                 with open(file_forwards, 'w+', encoding="utf-8") as forwards_file:
                                     c_forwards.to_csv(forwards_file, sep=';')
 
@@ -948,7 +985,7 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                                   + str(file_archive))
 
 
-                            if forward_check is True:
+                            if forwards_check is True:
                                 if forward_count >= 15:
                                     forwards_found = c_forwards.From.count()
                                     value_count = c_forwards['From'].value_counts()
@@ -1029,222 +1066,276 @@ def cli(target,comprehensive,media,forwards,user,location,alt,json):
                             else:
                                 pass
 
-                with client:
-                    client.loop.run_until_complete(main())
+                if user_check == True:
+                    my_user = None
+                    try:
+                        print(Fore.GREEN + ' [+] '
+                                         + Style.RESET_ALL
+                                         + 'User details for ' + t)
+                        user = int(t)
+                        my_user = await client.get_entity(PeerUser(int(user)))
 
-    if user:
+                        user_first_name = my_user.first_name
+                        user_last_name = my_user.last_name
+                        if user_last_name is not None:
+                            user_full_name = str(user_first_name) + ' ' + str(user_last_name)
+                        else:
+                            user_full_name = str(user_first_name)
 
-        client.connect()
-        if not client.is_user_authorized():
-            client.send_code_request(phone_number)
-            client.sign_in(phone_number)
-            try:
-                client.sign_in(code=input(' Enter code: '))
-            except SessionPasswordNeededError:
-                client.sign_in(password=getpass.getpass(
-                    prompt='Password: ', stream=None))
+                        if my_user.photo is not None:
+                            user_photo = my_user.photo.photo_id
+                        else:
+                            user_photo = 'None'
 
-        my_user = None
-        try:
-            print(Fore.GREEN + ' [+] '
-                             + Style.RESET_ALL
-                             + 'User details for ' + user)
-            user = int(user)
-            my_user = client.get_entity(PeerUser(int(user)))
-
-            user_first_name = my_user.first_name
-            user_last_name = my_user.last_name
-            if user_last_name is not None:
-                user_full_name = str(user_first_name) + ' ' + str(user_last_name)
-            else:
-                user_full_name = str(user_first_name)
-
-            if my_user.photo is not None:
-                user_photo = my_user.photo.photo_id
-            else:
-                user_photo = 'None'
-
-            if my_user.restriction_reason is not None:
-                ios_restriction = entity.restriction_reason[0]
-                if 1 in entity.restriction_reason:
-                    android_restriction = entity.restriction_reason[1]
-                    user_restrictions = str(
-                        ios_restriction) + ', ' + str(android_restriction)
-                else:
-                    user_restrictions = str(ios_restriction)
-            else:
-                user_restrictions = 'None'
+                        if my_user.restriction_reason is not None:
+                            ios_restriction = entity.restriction_reason[0]
+                            if 1 in entity.restriction_reason:
+                                android_restriction = entity.restriction_reason[1]
+                                user_restrictions = str(
+                                    ios_restriction) + ', ' + str(android_restriction)
+                            else:
+                                user_restrictions = str(ios_restriction)
+                        else:
+                            user_restrictions = 'None'
 
 
-            print(Fore.GREEN
-                  + '  ├  Username: '
-                  + Style.RESET_ALL
-                  + str(my_user.username))
-            print(Fore.GREEN
-                  + '  ├  Name: '
-                  + Style.RESET_ALL
-                  + str(user_full_name))
-            print(Fore.GREEN
-                  + '  ├  Verification: '
-                  + Style.RESET_ALL
-                  + str(my_user.verified))
-            print(Fore.GREEN
-                  + '  ├  Photo ID: '
-                  + Style.RESET_ALL
-                  + str(user_photo))
-            #print(Fore.GREEN
-            #      + '  ├  Status: '
-            #      + Style.RESET_ALL
-            #      + str(my_user.status))
-            print(Fore.GREEN
-                  + '  ├  Phone number: '
-                  + Style.RESET_ALL
-                  + str(my_user.phone))
-            print(Fore.GREEN
-                  + '  ├  Access hash: '
-                  + Style.RESET_ALL
-                  + str(my_user.access_hash))
-            print(Fore.GREEN
-                  + '  ├  Language: '
-                  + Style.RESET_ALL
-                  + str(my_user.lang_code))
-            print(Fore.GREEN
-                  + '  ├  Bot: '
-                  + Style.RESET_ALL
-                  + str(my_user.bot))
-            print(Fore.GREEN
-                  + '  ├  Scam: '
-                  + Style.RESET_ALL
-                  + str(my_user.scam))
-            print(Fore.GREEN
-                  + '  └  Restrictions: '
-                  + Style.RESET_ALL
-                  + str(user_restrictions))
+                        print(Fore.GREEN
+                              + '  ├  Username: '
+                              + Style.RESET_ALL
+                              + str(my_user.username))
+                        print(Fore.GREEN
+                              + '  ├  Name: '
+                              + Style.RESET_ALL
+                              + str(user_full_name))
+                        print(Fore.GREEN
+                              + '  ├  Verification: '
+                              + Style.RESET_ALL
+                              + str(my_user.verified))
+                        print(Fore.GREEN
+                              + '  ├  Photo ID: '
+                              + Style.RESET_ALL
+                              + str(user_photo))
+                        #print(Fore.GREEN
+                        #      + '  ├  Status: '
+                        #      + Style.RESET_ALL
+                        #      + str(my_user.status))
+                        print(Fore.GREEN
+                              + '  ├  Phone number: '
+                              + Style.RESET_ALL
+                              + str(my_user.phone))
+                        print(Fore.GREEN
+                              + '  ├  Access hash: '
+                              + Style.RESET_ALL
+                              + str(my_user.access_hash))
+                        print(Fore.GREEN
+                              + '  ├  Language: '
+                              + Style.RESET_ALL
+                              + str(my_user.lang_code))
+                        print(Fore.GREEN
+                              + '  ├  Bot: '
+                              + Style.RESET_ALL
+                              + str(my_user.bot))
+                        print(Fore.GREEN
+                              + '  ├  Scam: '
+                              + Style.RESET_ALL
+                              + str(my_user.scam))
+                        print(Fore.GREEN
+                              + '  └  Restrictions: '
+                              + Style.RESET_ALL
+                              + str(user_restrictions))
 
-        except ValueError:
-            pass
-        if my_user is None:
-            print(Fore.GREEN + ' [!] '
-                             + Style.RESET_ALL
-                             + 'User not found, this is likely because Telepathy has not encountered them yet.')
-
-    if location:
-
-        client.connect()
-        if not client.is_user_authorized():
-            client.send_code_request(phone_number)
-            client.sign_in(phone_number)
-            try:
-                client.sign_in(code=input(' Enter code: '))
-            except SessionPasswordNeededError:
-                client.sign_in(password=getpass.getpass(
-                    prompt='Password: ', stream=None))
-
-        print(Fore.GREEN + ' [!] '
-                         + Style.RESET_ALL
-                         + 'Searching for users near '
-                         + location
-                         + '\n')
-        latitude, longitude = location.split(sep=',')
-
-        locations_file = telepathy_file + 'locations/'
-
-        try:
-             os.makedirs(locations_file)
-        except FileExistsError:
-             pass
-
-        save_file = locations_file + latitude + '_' + longitude + '_' + 'locations.csv'
-
-        async def main():
-            locations_list = []
-            result = await client(functions.contacts.GetLocatedRequest(
-                 geo_point=types.InputGeoPoint(
-                     lat=float(latitude),
-                     long=float(longitude),
-                     accuracy_radius=42
-                 ),
-                 self_expires=42
-            ))
-
-            #progress bar?
-
-            for user in result.updates[0].peers:
-                try:
-                    user_df = pd.DataFrame(locations_list,
-                                              columns = ['User_ID', 'Distance'])
-                    if hasattr(user, 'peer'):
-                        ID = user.peer.user_id
-                    else:
-                        pass
-                    if hasattr(user, 'distance'):
-                        distance = user.distance
-                    else:
+                    except ValueError:
                         pass
+                    if my_user is None:
+                        print(Fore.GREEN + ' [!] '
+                                         + Style.RESET_ALL
+                                         + 'User not found, this is likely because Telepathy has not encountered them yet.')
 
-                    locations_list.append([ID, distance])
+                if location_check == True:
 
-                except:
-                    pass
+                    location = t
 
-            d_500 = 0
-            d_1000 = 0
-            d_2000 = 0
-            d_3000 = 0
+                    print(Fore.GREEN + ' [!] '
+                                     + Style.RESET_ALL
+                                     + 'Searching for users near '
+                                     + location
+                                     + '\n')
+                    latitude, longitude = location.split(sep=',')
 
-            for account, distance in user_df.itertuples(index=False):
-                account = int(account)
-                my_user = await client.get_entity(PeerUser(account))
-                user_id = my_user.id
-                name = my_user.first_name
-                distance = int(distance)
+                    locations_file = telepathy_file + 'locations/'
 
-                if distance == 500:
-                    d_500 += 1
-                elif distance == 1000:
-                    d_1000 += 1
-                elif distance == 2000:
-                    d_2000 += 1
-                elif distance == 3000:
-                    d_3000 += 1
+                    try:
+                         os.makedirs(locations_file)
+                    except FileExistsError:
+                         pass
+
+                    save_file = locations_file + latitude + '_' + longitude + '_' + 'locations.csv'
+
+                    locations_list = []
+                    result = await client(functions.contacts.GetLocatedRequest(
+                         geo_point=types.InputGeoPoint(
+                             lat=float(latitude),
+                             long=float(longitude),
+                             accuracy_radius=42
+                         ),
+                         self_expires=42
+                    ))
+
+                    #progress bar?
+
+                    for user in result.updates[0].peers:
+                        try:
+                            user_df = pd.DataFrame(locations_list,
+                                                      columns = ['User_ID', 'Distance'])
+                            if hasattr(user, 'peer'):
+                                ID = user.peer.user_id
+                            else:
+                                pass
+                            if hasattr(user, 'distance'):
+                                distance = user.distance
+                            else:
+                                pass
+
+                            locations_list.append([ID, distance])
+
+                        except:
+                            pass
+
+                    d_500 = 0
+                    d_1000 = 0
+                    d_2000 = 0
+                    d_3000 = 0
+
+                    for account, distance in user_df.itertuples(index=False):
+                        account = int(account)
+                        my_user = await client.get_entity(PeerUser(account))
+                        user_id = my_user.id
+                        name = my_user.first_name
+                        distance = int(distance)
+
+                        if distance == 500:
+                            d_500 += 1
+                        elif distance == 1000:
+                            d_1000 += 1
+                        elif distance == 2000:
+                            d_2000 += 1
+                        elif distance == 3000:
+                            d_3000 += 1
 
 
-            with open(save_file, 'w+', encoding="utf-8") as f: #could one day append, including access time to differentiate
-                user_df.to_csv(f, sep = ';', index = False)
+                    with open(save_file, 'w+', encoding="utf-8") as f: #could one day append, including access time to differentiate
+                        user_df.to_csv(f, sep = ';', index = False)
 
-            total = len(locations_list)
+                    total = len(locations_list)
 
-            print(Fore.GREEN
-                  + ' [+] Users located'
-                  + Style.RESET_ALL)
-            print(Fore.GREEN
-                  + '  ├  Users within 500m:  '
-                  + Style.RESET_ALL
-                  + str(d_500))
-            print(Fore.GREEN
-                  + '  ├  Users within 1000m: '
-                  + Style.RESET_ALL
-                  + str(d_1000))
-            print(Fore.GREEN
-                  + '  ├  Users within 2000m: '
-                  + Style.RESET_ALL
-                  + str(d_2000))
-            print(Fore.GREEN
-                  + '  ├  Users within 3000m: '
-                  + Style.RESET_ALL
-                  + str(d_3000))
-            print(Fore.GREEN
-                  + '  ├  Total users found:  '
-                  + Style.RESET_ALL
-                  + str(total))
-            print(Fore.GREEN
-                  + '  └  Location list saved to: '
-                  + Style.RESET_ALL
-                  + save_file)
+                    print(Fore.GREEN
+                          + ' [+] Users located'
+                          + Style.RESET_ALL)
+                    print(Fore.GREEN
+                          + '  ├  Users within 500m:  '
+                          + Style.RESET_ALL
+                          + str(d_500))
+                    print(Fore.GREEN
+                          + '  ├  Users within 1000m: '
+                          + Style.RESET_ALL
+                          + str(d_1000))
+                    print(Fore.GREEN
+                          + '  ├  Users within 2000m: '
+                          + Style.RESET_ALL
+                          + str(d_2000))
+                    print(Fore.GREEN
+                          + '  ├  Users within 3000m: '
+                          + Style.RESET_ALL
+                          + str(d_3000))
+                    print(Fore.GREEN
+                          + '  ├  Total users found:  '
+                          + Style.RESET_ALL
+                          + str(total))
+                    print(Fore.GREEN
+                          + '  └  Location list saved to: '
+                          + Style.RESET_ALL
+                          + save_file)
 
-            # can also do the same for channels with similar output file to users
+                    # can also do the same for channels with similar output file to users
+
+                    # may one day add trilateration to find users closest to exact point
+
+                if export == True:
+                    export_file = telepathy_file + 'export.csv'
+                    exports = []
+
+                    #progress bar
+
+                    for Dialog in await client.get_dialogs():
+                        try:
+                            if Dialog.entity.username:
+                                group_url = 'http://t.me/' + Dialog.entity.username
+                                group_username = Dialog.entity.username
+                                s = requests.Session()
+                                s.max_redirects = 10
+                                s.headers['User-Agent'] = random.choice(user_agent)
+                                URL = s.get(group_url)
+                                URL.encoding = 'utf-8'
+                                html_content = URL.text
+                                soup = BeautifulSoup(html_content, 'html.parser')
+                                name = Dialog.entity.title
+                                try:
+                                    group_description = soup.find(
+                                        'div', {'class': ['tgme_page_description']}).text
+                                    descript = Fore.GREEN + 'Description: ' + Style.RESET_ALL
+                                    prefix = descript + ' '
+                                    preferredWidth = 70
+                                    wrapper_d = textwrap.TextWrapper(initial_indent = descript,
+                                                                     width = preferredWidth,
+                                                                     subsequent_indent = '                  ')
+                                except:
+                                    group_description = "None"
+                                    descript = Fore.GREEN + 'Description: ' + Style.RESET_ALL
+                                    prefix = descript + ' '
+                                    preferredWidth = 70
+                                    wrapper_d = textwrap.TextWrapper(initial_indent = descript,
+                                                                     width = preferredWidth,
+                                                                     subsequent_indent = '                  ')
+                                try:
+                                    group_participants = soup.find('div', {'class': ['tgme_page_extra']}).text
+                                    sep = 'members'
+                                    stripped = group_participants.split(sep, 1)[0]
+                                    total_participants = stripped.replace(' ','').replace('members','').replace('subscribers','').replace('member','')
+                                except:
+                                    total_participants = 'Not found' # could be due to restriction, might need to mention
+
+                                if Dialog.entity.broadcast is True:
+                                    chat_type = 'Channel'
+                                elif Dialog.entity.megagroup is True:
+                                    chat_type = 'Megagroup'
+                                elif Dialog.entity.gigagroup is True:
+                                    chat_type = 'Gigagroup'
+                                else:
+                                    chat_type = 'Chat'
+
+                                if Dialog.entity.restriction_reason is not None:
+                                    ios_restriction = Dialog.entity.restriction_reason[0]
+                                    if 1 in Dialog.entity.restriction_reason:
+                                        android_restriction =Dialog.entity.restriction_reason[1]
+                                        group_status = str(
+                                            ios_restriction) + ', ' + str(android_restriction)
+                                    else:
+                                        group_status = str(ios_restriction)
+                                else:
+                                    group_status = 'None'
+
+                                exports.append([filetime,Dialog.entity.title,group_description,total_participants,group_username,group_url,chat_type,Dialog.entity.id,Dialog.entity.access_hash,group_status])
+
+                                export_df = pd.DataFrame(exports, columns = ['Access Date','Title','Description','Total participants','Username','URL','Chat type','Chat ID','Access hash','Restrictions'])
+
+                                if not os.path.isfile(export_file):
+                                   export_df.to_csv(export_file, sep = ';', index = False)
+                                else:
+                                   export_df.to_csv(export_file, sep = ';', mode='w', index = False)
+
+                        except AttributeError:
+                            pass
 
-            # may one day add trilateration to find users closest to exact point
 
         with client:
             client.loop.run_until_complete(main())
